@@ -182,21 +182,39 @@ func (c *ClusterChecker) syncClusterToNodes(ctx context.Context) error {
 	if err != nil {
 		return err
 	}
+	version := clusterInfo.Version.Load()
 	for _, shard := range clusterInfo.Shards {
 		for _, node := range shard.Nodes {
-			// sync the clusterName to the latest version
-			if err := node.SyncClusterInfo(ctx, clusterInfo); err != nil {
-				return err
-			}
+			go func(n store.Node) {
+				log := logger.Get().With(
+					zap.String("namespace", c.namespace),
+					zap.String("cluster", c.clusterName),
+					zap.Int64("version", version),
+					zap.String("node_id", n.ID()),
+					zap.String("addr", n.Addr()))
+				// sync the clusterName to the latest version
+				if err := n.SyncClusterInfo(ctx, clusterInfo); err != nil {
+					log.Error("Failed to sync the cluster topology to the node", zap.Error(err))
+				} else {
+					log.Info("Succeed to sync the cluster topology to the node")
+				}
+			}(node)
 		}
 	}
 	return nil
 }
 
 func (c *ClusterChecker) parallelProbeNodes(ctx context.Context, cluster *store.Cluster) {
+	var mu sync.Mutex
+	var latestNodeVersion int64 = 0
+	var latestClusterNodesStr string
+	var wg sync.WaitGroup
+
 	for i, shard := range cluster.Shards {
 		for _, node := range shard.Nodes {
+			wg.Add(1)
 			go func(shardIdx int, n store.Node) {
+				defer wg.Done()
 				log := logger.Get().With(
 					zap.String("id", n.ID()),
 					zap.Bool("is_master", n.IsMaster()),
@@ -223,16 +241,46 @@ func (c *ClusterChecker) parallelProbeNodes(ctx context.Context, cluster *store.
 					if err := n.SyncClusterInfo(ctx, cluster); err != nil {
 						log.With(zap.Error(err)).Error("Failed to sync the clusterName info")
 					}
-				} else if version > cluster.Version.Load() {
+				} else if version > clusterVersion {
 					log.With(
 						zap.Int64("node.version", version),
 						zap.Int64("clusterName.version", clusterVersion),
 					).Warn("The node is in a higher version")
+					mu.Lock()
+					if version > latestNodeVersion {
+						latestNodeVersion = version
+						clusterNodesStr, errX := n.GetClusterNodesString(ctx)
+						if errX != nil {
+							log.With(zap.String("node", n.ID()), zap.Error(errX)).Error("Failed to get the cluster nodes info from node")
+							// set empty explicitly
+							latestClusterNodesStr = ""
+						} else {
+							latestClusterNodesStr = clusterNodesStr
+						}
+					}
+					mu.Unlock()
 				}
 				c.resetFailureCount(n.ID())
 			}(i, node)
 		}
 	}
+
+	wg.Wait()
+	if latestNodeVersion > cluster.Version.Load() && latestClusterNodesStr != "" {
+		latestClusterInfo, err := store.ParseCluster(latestClusterNodesStr)
+		if err != nil {
+			logger.Get().With(zap.String("cluster", latestClusterNodesStr), zap.Error(err)).Error("Failed to parse the cluster info")
+			return
+		}
+		latestClusterInfo.Name = cluster.Name
+		latestClusterInfo.SetPassword(cluster.Shards[0].Nodes[0].Password())
+		err = c.clusterStore.SetCluster(ctx, c.namespace, latestClusterInfo)
+		if err != nil {
+			logger.Get().With(zap.String("cluster", latestClusterNodesStr), zap.Error(err)).Error("Failed to update the cluster info")
+			return
+		}
+		logger.Get().With(zap.Any("latestClusterInfo", latestClusterInfo)).Info("Refresh latest cluster info to all nodes")
+	}
 }
 
 func (c *ClusterChecker) probeLoop() {
@@ -281,50 +329,61 @@ func (c *ClusterChecker) tryUpdateMigrationStatus(ctx context.Context, clonedClu
 		if !shard.IsMigrating() {
 			continue
 		}
-
-		sourceNodeClusterInfo, err := shard.GetMasterNode().GetClusterInfo(ctx)
+		sourceNode := shard.GetMasterNode()
+		sourceNodeClusterInfo, err := sourceNode.GetClusterInfo(ctx)
 		if err != nil {
-			log.Error("Failed to get the cluster info from the source node", zap.Error(err))
-			return
+			log.With(
+				zap.Int("shard_index", i),
+				zap.String("source_node", sourceNode.ID()),
+			).Error("Failed to get the cluster info from the source node", zap.Error(err))
+			continue
 		}
-		if sourceNodeClusterInfo.MigratingSlot != shard.MigratingSlot {
-			log.Error("Mismatch migrate slot", zap.Int("slot", shard.MigratingSlot))
-			return
+		if !sourceNodeClusterInfo.MigratingSlot.Equal(shard.MigratingSlot.SlotRange) {
+			log.Error("Mismatch migrating slot",
+				zap.Int("shard_index", i),
+				zap.String("source_migrating_slot", sourceNodeClusterInfo.MigratingSlot.String()),
+				zap.String("migrating_slot", shard.MigratingSlot.String()),
+			)
+			continue
 		}
 		if shard.TargetShardIndex < 0 || shard.TargetShardIndex >= len(clonedCluster.Shards) {
 			log.Error("Invalid target shard index", zap.Int("index", shard.TargetShardIndex))
 			return
 		}
-		targetMasterNode := clonedCluster.Shards[shard.TargetShardIndex].GetMasterNode()
 
 		switch sourceNodeClusterInfo.MigratingState {
 		case "none", "start":
 			continue
 		case "fail":
+			migratingSlot := shard.MigratingSlot
 			clonedCluster.Shards[i].ClearMigrateState()
 			if err := c.clusterStore.UpdateCluster(ctx, c.namespace, clonedCluster); err != nil {
 				log.Error("Failed to update the cluster", zap.Error(err))
 				return
 			}
 			c.updateCluster(clonedCluster)
-			log.Warn("Failed to migrate the slot", zap.Int("slot", shard.MigratingSlot))
+			log.Warn("Failed to migrate the slot", zap.String("slot", migratingSlot.String()))
 		case "success":
-			err := clonedCluster.SetSlot(ctx, shard.MigratingSlot, targetMasterNode.ID())
-			if err != nil {
-				log.Error("Failed to set the slot", zap.Error(err))
-				return
-			}
-			clonedCluster.Shards[i].SlotRanges = store.RemoveSlotFromSlotRanges(clonedCluster.Shards[i].SlotRanges, shard.MigratingSlot)
+			clonedCluster.Shards[i].SlotRanges = store.RemoveSlotFromSlotRanges(clonedCluster.Shards[i].SlotRanges, shard.MigratingSlot.SlotRange)
 			clonedCluster.Shards[shard.TargetShardIndex].SlotRanges = store.AddSlotToSlotRanges(
-				clonedCluster.Shards[shard.TargetShardIndex].SlotRanges, shard.MigratingSlot)
+				clonedCluster.Shards[shard.TargetShardIndex].SlotRanges, shard.MigratingSlot.SlotRange,
+			)
+			migratedSlot := shard.MigratingSlot
 			clonedCluster.Shards[i].ClearMigrateState()
 			if err := c.clusterStore.UpdateCluster(ctx, c.namespace, clonedCluster); err != nil {
 				log.Error("Failed to update the cluster", zap.Error(err))
+				return
 			} else {
-				log.Info("Migrate the slot successfully", zap.Int("slot", shard.MigratingSlot))
+				log.Info("Migrate the slot successfully", zap.String("slot", migratedSlot.String()))
 			}
 			c.updateCluster(clonedCluster)
 		default:
+			clonedCluster.Shards[i].ClearMigrateState()
+			if err := c.clusterStore.UpdateCluster(ctx, c.namespace, clonedCluster); err != nil {
+				log.Error("Failed to update the cluster", zap.Error(err))
+				return
+			}
+			c.updateCluster(clonedCluster)
 			log.Error("Unknown migrating state", zap.String("state", sourceNodeClusterInfo.MigratingState))
 		}
 	}
