@@ -20,13 +20,16 @@
 
 package api
 
-import "github.com/apache/kvrocks-controller/store"
+import (
+	"github.com/apache/kvrocks-controller/store"
+)
 
 type Handler struct {
 	Namespace *NamespaceHandler
 	Cluster   *ClusterHandler
 	Shard     *ShardHandler
 	Node      *NodeHandler
+	Raft      *RaftHandler
 }
 
 func NewHandler(s *store.ClusterStore) *Handler {
@@ -35,5 +38,6 @@ func NewHandler(s *store.ClusterStore) *Handler {
 		Cluster:   &ClusterHandler{s: s},
 		Shard:     &ShardHandler{s: s},
 		Node:      &NodeHandler{s: s},
+		Raft:      &RaftHandler{},
 	}
 }
