@@ -27,12 +27,17 @@ import (
 	"net/http"
 	"net/http/httptest"
 	"strconv"
+	"strings"
 	"testing"
+	"time"
 
 	"github.com/gin-gonic/gin"
+	"github.com/go-redis/redis/v8"
 	"github.com/stretchr/testify/require"
 
+	"github.com/apache/kvrocks-controller/config"
 	"github.com/apache/kvrocks-controller/consts"
+	"github.com/apache/kvrocks-controller/controller"
 	"github.com/apache/kvrocks-controller/server/middleware"
 	"github.com/apache/kvrocks-controller/store"
 	"github.com/apache/kvrocks-controller/store/engine"
@@ -84,7 +89,8 @@ func TestShardBasics(t *testing.T) {
 		ctx.Params = []gin.Param{
 			{Key: "namespace", Value: ns},
 			{Key: "cluster", Value: clusterName},
-			{Key: "shard", Value: strconv.Itoa(shardIndex)}}
+			{Key: "shard", Value: strconv.Itoa(shardIndex)},
+		}
 
 		middleware.RequiredClusterShard(ctx)
 		require.Equal(t, http.StatusOK, recorder.Code)
@@ -103,7 +109,8 @@ func TestShardBasics(t *testing.T) {
 		ctx.Params = []gin.Param{
 			{Key: "namespace", Value: ns},
 			{Key: "cluster", Value: clusterName},
-			{Key: "shard", Value: "1"}}
+			{Key: "shard", Value: "1"},
+		}
 
 		middleware.RequiredClusterShard(ctx)
 		require.Equal(t, http.StatusOK, recorder.Code)
@@ -124,7 +131,7 @@ func TestShardBasics(t *testing.T) {
 			nodeAddrs = append(nodeAddrs, node.Addr())
 		}
 		require.ElementsMatch(t, []string{"127.0.0.1:1235", "127.0.0.1:1236"}, nodeAddrs)
-		require.EqualValues(t, -1, rsp.Data.Shard.MigratingSlot)
+		require.Nil(t, rsp.Data.Shard.MigratingSlot)
 		require.EqualValues(t, -1, rsp.Data.Shard.TargetShardIndex)
 	})
 
@@ -159,11 +166,30 @@ func TestShardBasics(t *testing.T) {
 func TestClusterFailover(t *testing.T) {
 	ns := "test-ns"
 	clusterName := "test-cluster-failover"
-	handler := &ShardHandler{s: store.NewClusterStore(engine.NewMock())}
+	clusterStore := store.NewClusterStore(engine.NewMock())
+	handler := &ShardHandler{s: clusterStore}
 	cluster, err := store.NewCluster(clusterName, []string{"127.0.0.1:7770", "127.0.0.1:7771"}, 2)
 	require.NoError(t, err)
 	node0, _ := cluster.Shards[0].Nodes[0].(*store.ClusterNode)
 	node1, _ := cluster.Shards[0].Nodes[1].(*store.ClusterNode)
+	masterClient := redis.NewClient(&redis.Options{Addr: node0.Addr()})
+	slaveClient := redis.NewClient(&redis.Options{Addr: node1.Addr()})
+
+	ctx := context.Background()
+
+	require.NoError(t, cluster.Reset(ctx))
+	require.NoError(t, cluster.SyncToNodes(ctx))
+	defer func() {
+		require.NoError(t, cluster.Reset(ctx))
+	}()
+
+	ctrl, err := controller.New(clusterStore, &config.ControllerConfig{
+		FailOver: &config.FailOverConfig{MaxPingCount: 3, PingIntervalSeconds: 3},
+	})
+	require.NoError(t, err)
+	require.NoError(t, ctrl.Start(ctx))
+	ctrl.WaitForReady()
+	defer ctrl.Close()
 
 	runFailover := func(t *testing.T, shardIndex, expectedStatusCode int) {
 		recorder := httptest.NewRecorder()
@@ -172,7 +198,8 @@ func TestClusterFailover(t *testing.T) {
 		ctx.Params = []gin.Param{
 			{Key: "namespace", Value: ns},
 			{Key: "cluster", Value: clusterName},
-			{Key: "shard", Value: strconv.Itoa(shardIndex)}}
+			{Key: "shard", Value: strconv.Itoa(shardIndex)},
+		}
 
 		middleware.RequiredClusterShard(ctx)
 		require.Equal(t, http.StatusOK, recorder.Code)
@@ -181,18 +208,38 @@ func TestClusterFailover(t *testing.T) {
 	}
 
 	t.Run("failover is good", func(t *testing.T) {
-		ctx := context.Background()
-		masterCli := node0.GetClient()
-		replicaCli := node1.GetClient()
-		require.NoError(t, masterCli.ClusterResetHard(ctx).Err())
-		require.NoError(t, replicaCli.ClusterResetHard(ctx).Err())
-		defer func() {
-			require.NoError(t, masterCli.ClusterResetHard(ctx).Err())
-			require.NoError(t, replicaCli.ClusterResetHard(ctx).Err())
-		}()
-
 		require.NoError(t, handler.s.CreateCluster(ctx, ns, cluster))
+		require.Eventually(t, func() bool {
+			// Confirm that the cluster info has been synced to each node
+			clusterInfo, err := node0.GetClusterInfo(ctx)
+			if err != nil {
+				return false
+			}
+			return clusterInfo.CurrentEpoch >= 1
+		}, 10*time.Second, 100*time.Millisecond)
+		require.NoError(t, masterClient.Set(ctx, "my_key", 100, 0).Err())
+		require.Eventually(t, func() bool {
+			info := strings.Split(slaveClient.Info(ctx).Val(), "\r\n")
+
+			var role string
+			sequence := 0
+			for _, line := range info {
+				kv := strings.Split(line, ":")
+				if len(kv) < 2 {
+					continue
+				}
+				if kv[0] == "role" {
+					role = kv[1]
+				}
+				if kv[0] == "sequence" {
+					sequence, err = strconv.Atoi(kv[1])
+				}
+			}
+			return role == "slave" && sequence > 0
+		}, 30*time.Second, 100*time.Millisecond)
+
 		runFailover(t, 0, http.StatusOK)
+		require.NoError(t, slaveClient.FlushAll(ctx).Err())
 	})
 
 	t.Run("cluster topology is good", func(t *testing.T) {
@@ -221,4 +268,18 @@ func TestClusterFailover(t *testing.T) {
 		require.NoError(t, err)
 		require.EqualValues(t, "master", clusterNodeInfo1.Role)
 	})
+
+	t.Run("failover with invalid node id", func(t *testing.T) {
+		recorder := httptest.NewRecorder()
+		ctx := GetTestContext(recorder)
+		ctx.Set(consts.ContextKeyStore, handler.s)
+		ctx.Params = []gin.Param{
+			{Key: "namespace", Value: ns},
+			{Key: "cluster", Value: clusterName},
+			{Key: "shard", Value: "0"},
+		}
+		ctx.Request.Body = io.NopCloser(bytes.NewBufferString(`{"preferred_node_id": "1234567890"}`))
+		middleware.RequiredClusterShard(ctx)
+		require.Equal(t, http.StatusOK, recorder.Code)
+	})
 }
