@@ -33,11 +33,17 @@ import (
 	"github.com/apache/kvrocks-controller/consts"
 )
 
+const (
+	// the old migrating slot was denoted by an int and -1 was
+	// used to denote a non migrating slot
+	NotMigratingInt = -1
+)
+
 type Shard struct {
-	Nodes            []Node      `json:"nodes"`
-	SlotRanges       []SlotRange `json:"slot_ranges"`
-	TargetShardIndex int         `json:"target_shard_index"`
-	MigratingSlot    int         `json:"migrating_slot"`
+	Nodes            []Node         `json:"nodes"`
+	SlotRanges       []SlotRange    `json:"slot_ranges"`
+	TargetShardIndex int            `json:"target_shard_index"`
+	MigratingSlot    *MigratingSlot `json:"migrating_slot"`
 }
 
 type Shards []*Shard
@@ -45,9 +51,11 @@ type Shards []*Shard
 func (s Shards) Len() int {
 	return len(s)
 }
+
 func (s Shards) Swap(i, j int) {
 	s[i], s[j] = s[j], s[i]
 }
+
 func (s Shards) Less(i, j int) bool {
 	if len(s[i].SlotRanges) == 0 {
 		return false
@@ -61,7 +69,7 @@ func NewShard() *Shard {
 	return &Shard{
 		Nodes:            make([]Node, 0),
 		SlotRanges:       make([]SlotRange, 0),
-		MigratingSlot:    -1,
+		MigratingSlot:    nil,
 		TargetShardIndex: -1,
 	}
 }
@@ -78,7 +86,7 @@ func (shard *Shard) Clone() *Shard {
 }
 
 func (shard *Shard) ClearMigrateState() {
-	shard.MigratingSlot = -1
+	shard.MigratingSlot = nil
 	shard.TargetShardIndex = -1
 }
 
@@ -91,26 +99,26 @@ func (shard *Shard) IsServicing() bool {
 	return shard.IsMigrating()
 }
 
-func (shard *Shard) addNode(addr, role, password string) error {
+func (shard *Shard) addNode(addr, role, password string) (*ClusterNode, error) {
 	if role != RoleMaster && role != RoleSlave {
-		return fmt.Errorf("%w: role", consts.ErrInvalidArgument)
+		return nil, fmt.Errorf("%w: role", consts.ErrInvalidArgument)
 	}
 	for _, node := range shard.Nodes {
 		if node.Addr() == addr {
-			return consts.ErrAlreadyExists
+			return nil, consts.ErrAlreadyExists
 		}
 	}
 	if role == RoleMaster && len(shard.Nodes) > 0 {
-		return fmt.Errorf("master node %w", consts.ErrAlreadyExists)
+		return nil, fmt.Errorf("master node %w", consts.ErrAlreadyExists)
 	}
 	node := NewClusterNode(addr, password)
 	node.SetRole(role)
 	shard.Nodes = append(shard.Nodes, node)
-	return nil
+	return node, nil
 }
 
 func (shard *Shard) IsMigrating() bool {
-	return shard.MigratingSlot != -1 && shard.TargetShardIndex != -1
+	return shard.MigratingSlot != nil && shard.MigratingSlot.IsMigrating && shard.TargetShardIndex != -1
 }
 
 func (shard *Shard) GetMasterNode() Node {
@@ -148,7 +156,8 @@ func (shard *Shard) getNewMasterNodeIndex(ctx context.Context, masterNodeIndex i
 		if i == masterNodeIndex {
 			continue
 		}
-		clusterNodeInfo, err := node.GetClusterNodeInfo(ctx)
+
+		_, err := node.GetClusterInfo(ctx)
 		if err != nil {
 			logger.Get().With(
 				zap.Error(err),
@@ -157,6 +166,33 @@ func (shard *Shard) getNewMasterNodeIndex(ctx context.Context, masterNodeIndex i
 			).Warn("Skip the node due to failed to get cluster info")
 			continue
 		}
+
+		clusterNodeInfo, err := node.GetClusterNodeInfo(ctx)
+		if err != nil {
+			logger.Get().With(
+				zap.Error(err),
+				zap.String("id", node.ID()),
+				zap.String("addr", node.Addr()),
+			).Warn("Skip the node due to failed to get info of node")
+			continue
+		}
+		if clusterNodeInfo.Role != RoleSlave || clusterNodeInfo.Sequence == 0 {
+			logger.Get().With(
+				zap.String("id", node.ID()),
+				zap.String("addr", node.Addr()),
+				zap.String("role", clusterNodeInfo.Role),
+				zap.Uint64("sequence", clusterNodeInfo.Sequence),
+			).Warn("Skip the node due to role or sequence invalid")
+			continue
+		}
+
+		logger.Get().With(
+			zap.String("id", node.ID()),
+			zap.String("addr", node.Addr()),
+			zap.String("role", clusterNodeInfo.Role),
+			zap.Uint64("sequence", clusterNodeInfo.Sequence),
+		).Info("Get slave node info successfully")
+
 		// If the preferredNodeID is not empty, we will use it as the new master node.
 		if preferredNodeID != "" && node.ID() == preferredNodeID {
 			newMasterNodeIndex = i
@@ -204,7 +240,7 @@ func (shard *Shard) promoteNewMaster(ctx context.Context, masterNodeID, preferre
 	return preferredNewMasterNode.ID(), nil
 }
 
-func (shard *Shard) HasOverlap(slotRange *SlotRange) bool {
+func (shard *Shard) HasOverlap(slotRange SlotRange) bool {
 	for _, shardSlotRange := range shard.SlotRanges {
 		if shardSlotRange.HasOverlap(slotRange) {
 			return true
@@ -259,7 +295,7 @@ func (shard *Shard) UnmarshalJSON(bytes []byte) error {
 	var data struct {
 		SlotRanges       []SlotRange    `json:"slot_ranges"`
 		TargetShardIndex int            `json:"target_shard_index"`
-		MigratingSlot    int            `json:"migrating_slot"`
+		MigratingSlot    *MigratingSlot `json:"migrating_slot"`
 		Nodes            []*ClusterNode `json:"nodes"`
 	}
 	if err := json.Unmarshal(bytes, &data); err != nil {
