@@ -29,11 +29,11 @@ import (
 
 func TestShard_HasOverlap(t *testing.T) {
 	shard := NewShard()
-	slotRange := &SlotRange{Start: 0, Stop: 100}
-	shard.SlotRanges = append(shard.SlotRanges, *slotRange)
+	slotRange := SlotRange{Start: 0, Stop: 100}
+	shard.SlotRanges = append(shard.SlotRanges, slotRange)
 	require.True(t, shard.HasOverlap(slotRange))
-	require.True(t, shard.HasOverlap(&SlotRange{Start: 50, Stop: 150}))
-	require.False(t, shard.HasOverlap(&SlotRange{Start: 101, Stop: 150}))
+	require.True(t, shard.HasOverlap(SlotRange{Start: 50, Stop: 150}))
+	require.False(t, shard.HasOverlap(SlotRange{Start: 101, Stop: 150}))
 }
 
 func TestShard_Sort(t *testing.T) {
@@ -54,17 +54,24 @@ func TestShard_Sort(t *testing.T) {
 }
 
 func TestShard_IsServicing(t *testing.T) {
+	var err error
 	shard := NewShard()
 	shard.TargetShardIndex = 0
-	shard.MigratingSlot = -1
+	shard.MigratingSlot = &MigratingSlot{IsMigrating: false}
 	require.False(t, shard.IsServicing())
 
 	shard.TargetShardIndex = 0
-	shard.MigratingSlot = 0
+	shard.MigratingSlot = nil
+	require.False(t, shard.IsServicing())
+
+	shard.TargetShardIndex = 0
+	slotRange, err := NewSlotRange(1, 1)
+	require.Nil(t, err)
+	shard.MigratingSlot = FromSlotRange(slotRange)
 	require.True(t, shard.IsServicing())
 
 	shard.TargetShardIndex = -1
-	shard.MigratingSlot = -1
+	shard.MigratingSlot = &MigratingSlot{IsMigrating: false}
 	shard.SlotRanges = []SlotRange{{Start: 0, Stop: 100}}
 	require.True(t, shard.IsServicing())
 
