@@ -23,9 +23,26 @@ const apiPrefix = "/api/v1";
 const apiHost = `${apiPrefix}`;
 
 export interface Cluster {
-  name: string;
-  version: number;
-  shards: {};
+    name: string;
+    version: number;
+    shards: {};
+}
+
+// Helper methods to handle common response patterns
+function handleCreateResponse(responseData: any): string {
+    if (responseData?.data != undefined) {
+        return "";
+    } else {
+        return handleError(responseData);
+    }
+}
+
+function handleDeleteResponse(responseData: any): string {
+    if (responseData == null || responseData.data == null || responseData.data === "ok") {
+        return "";
+    } else {
+        return handleError(responseData);
+    }
 }
 
 export async function fetchNamespaces(): Promise<string[]> {
@@ -43,11 +60,7 @@ export async function createNamespace(name: string): Promise<string> {
         const { data: responseData } = await axios.post(`${apiHost}/namespaces`, {
             namespace: name,
         });
-        if (responseData?.data != undefined) {
-            return "";
-        } else {
-            return handleError(responseData);
-        }
+        return handleCreateResponse(responseData);
     } catch (error) {
         return handleError(error);
     }
@@ -55,14 +68,8 @@ export async function createNamespace(name: string): Promise<string> {
 
 export async function deleteNamespace(name: string): Promise<string> {
     try {
-        const { data: responseData } = await axios.delete(
-            `${apiHost}/namespaces/${name}`
-        );
-        if (responseData.data == null) {
-            return "";
-        } else {
-            return handleError(responseData);
-        }
+        const { data: responseData } = await axios.delete(`${apiHost}/namespaces/${name}`);
+        return handleDeleteResponse(responseData);
     } catch (error) {
         return handleError(error);
     }
@@ -78,13 +85,14 @@ export async function createCluster(
     try {
         const { data: responseData } = await axios.post(
             `${apiHost}/namespaces/${namespace}/clusters`,
-            { name, nodes, replicas, password }
+            {
+                name,
+                nodes,
+                replicas,
+                password,
+            }
         );
-        if (responseData?.data != undefined) {
-            return "";
-        } else {
-            return handleError(responseData);
-        }
+        return handleCreateResponse(responseData);
     } catch (error) {
         return handleError(error);
     }
@@ -102,10 +110,7 @@ export async function fetchClusters(namespace: string): Promise<string[]> {
     }
 }
 
-export async function fetchCluster(
-    namespace: string,
-    cluster: string
-): Promise<Object> {
+export async function fetchCluster(namespace: string, cluster: string): Promise<Object> {
     try {
         const { data: responseData } = await axios.get(
             `${apiHost}/namespaces/${namespace}/clusters/${cluster}`
@@ -117,19 +122,12 @@ export async function fetchCluster(
     }
 }
 
-export async function deleteCluster(
-    namespace: string,
-    cluster: string
-): Promise<string> {
+export async function deleteCluster(namespace: string, cluster: string): Promise<string> {
     try {
         const { data: responseData } = await axios.delete(
             `${apiHost}/namespaces/${namespace}/clusters/${cluster}`
         );
-        if (responseData.data == null) {
-            return "";
-        } else {
-            return handleError(responseData);
-        }
+        return handleDeleteResponse(responseData);
     } catch (error) {
         return handleError(error);
     }
@@ -147,11 +145,7 @@ export async function importCluster(
             { nodes, password }
         );
         console.log("importCluster response", responseData);
-        if (responseData?.data != undefined) {
-            return "";
-        } else {
-            return handleError(responseData);
-        }
+        return handleCreateResponse(responseData);
     } catch (error) {
         return handleError(error);
     }
@@ -161,7 +155,7 @@ export async function migrateSlot(
     namespace: string,
     cluster: string,
     target: number,
-    slot: number,
+    slot: string,
     slotOnly: boolean
 ): Promise<string> {
     try {
@@ -173,11 +167,7 @@ export async function migrateSlot(
                 slot_only: slotOnly,
             }
         );
-        if (responseData?.data != undefined) {
-            return "";
-        } else {
-            return handleError(responseData);
-        }
+        return handleCreateResponse(responseData);
     } catch (error) {
         return handleError(error);
     }
@@ -194,11 +184,7 @@ export async function createShard(
             `${apiHost}/namespaces/${namespace}/clusters/${cluster}/shards`,
             { nodes, password }
         );
-        if (responseData?.data != undefined) {
-            return "";
-        } else {
-            return handleError(responseData);
-        }
+        return handleCreateResponse(responseData);
     } catch (error) {
         return handleError(error);
     }
@@ -220,10 +206,7 @@ export async function fetchShard(
     }
 }
 
-export async function listShards(
-    namespace: string,
-    cluster: string
-): Promise<Object[]> {
+export async function listShards(namespace: string, cluster: string): Promise<Object[]> {
     try {
         const { data: responseData } = await axios.get(
             `${apiHost}/namespaces/${namespace}/clusters/${cluster}/shards`
@@ -244,11 +227,7 @@ export async function deleteShard(
         const { data: responseData } = await axios.delete(
             `${apiHost}/namespaces/${namespace}/clusters/${cluster}/shards/${shard}`
         );
-        if (responseData.data == null) {
-            return "";
-        } else {
-            return handleError(responseData);
-        }
+        return handleDeleteResponse(responseData);
     } catch (error) {
         return handleError(error);
     }
@@ -267,11 +246,7 @@ export async function createNode(
             `${apiHost}/namespaces/${namespace}/clusters/${cluster}/shards/${shard}/nodes`,
             { addr, role, password }
         );
-        if (responseData?.data == null) {
-            return "";
-        } else {
-            return handleError(responseData);
-        }
+        return handleCreateResponse(responseData);
     } catch (error) {
         return handleError(error);
     }
@@ -303,17 +278,40 @@ export async function deleteNode(
         const { data: responseData } = await axios.delete(
             `${apiHost}/namespaces/${namespace}/clusters/${cluster}/shards/${shard}/nodes/${nodeId}`
         );
-        if (responseData.data == null) {
-            return "";
-        } else {
-            return handleError(responseData);
-        }
+        return handleDeleteResponse(responseData);
     } catch (error) {
         console.log(error);
         return handleError(error);
     }
 }
 
+export async function failoverShard(
+    namespace: string,
+    cluster: string,
+    shard: string,
+    preferredNodeId?: string
+): Promise<{ newMasterId?: string; error?: string }> {
+    try {
+        const requestBody: { preferred_node_id?: string } = {};
+        if (preferredNodeId) {
+            requestBody.preferred_node_id = preferredNodeId;
+        }
+
+        const { data: responseData } = await axios.post(
+            `${apiHost}/namespaces/${namespace}/clusters/${cluster}/shards/${shard}/failover`,
+            requestBody
+        );
+
+        if (responseData?.data?.new_master_id) {
+            return { newMasterId: responseData.data.new_master_id };
+        } else {
+            return { error: handleError(responseData) };
+        }
+    } catch (error) {
+        return { error: handleError(error) };
+    }
+}
+
 function handleError(error: any): string {
     let message: string = "";
     if (error instanceof AxiosError) {
